/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.s3unit.helper;

import javax.jdo.PersistenceManager;

import org.slim3.jdo.CurrentPersistenceManager;
import org.slim3.jdo.ModelMeta;
import org.slim3.jdo.SelectQuery;

/**
 * A helper for JDO tests.
 * 
 * @author bufferings[at]gmail.com
 * @see org.slim3.tester.JDOTestCase
 */
public class JDOHelper {

  /**
   * a persistence manager
   */
  protected PersistenceManager pm;

  /**
   * Sets up before the Test method.
   */
  public void setUp() {
    pm = CurrentPersistenceManager.getProxy();
  }

  /**
   * Tears down after the Test method.
   */
  public void tearDown() {
    pm = null;
  }

  /**
   * Gets the persistence manager.
   * 
   * @return pm the persistence manager
   */
  public PersistenceManager getPm() {
    return pm;
  }

  /**
   * Creates a new {@link SelectQuery}.
   * 
   * @param <M>
   *          the model type
   * @param modelMeta
   *          the meta data of model
   * @return a new {@link SelectQuery}
   */
  public <M> SelectQuery<M> from(ModelMeta<M> modelMeta) {
    return new SelectQuery<M>(pm, modelMeta.getModelClass());
  }

  /**
   * Creates a new {@link SelectQuery}.
   * 
   * @param <M>
   *          the model type
   * @param modelClass
   *          the model class
   * @return a new {@link SelectQuery}
   */
  public <M> SelectQuery<M> from(Class<M> modelClass) {
    return new SelectQuery<M>(pm, modelClass);
  }

  /**
   * Makes the model persistent in transaction.
   * 
   * @param <T>
   *          the model type
   * @param model
   *          the model
   * @return the persistent model
   */
  public <T> T makePersistentInTx(T model) {
    begin();
    T t = pm.makePersistent(model);
    commit();
    return t;
  }

  /**
   * Deletes the persistent model from the data store in transaction.
   * 
   * @param model
   *          the model
   */
  public void deletePersistentInTx(Object model) {
    begin();
    pm.deletePersistent(model);
    commit();
  }

  /**
   * Begins the current transaction.
   */
  public void begin() {
    pm.currentTransaction().begin();
  }

  /**
   * Rolls back the current transaction.
   */
  public void commit() {
    pm.currentTransaction().commit();
  }

  /**
   * Rolls back transaction.
   */
  public void rollback() {
    if (pm.currentTransaction().isActive()) {
      pm.currentTransaction().rollback();
    }
  }

  /**
   * Reopens the current persistence manager.
   */
  public void reopenPersistenceManager() {
    CurrentPersistenceManager.reopen();
  }
}
