/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.s3unit.util;

import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Logger;

import bufferings.ktr.s3unit.exception.KtrS3UnitException;

import com.google.apphosting.api.ApiProxy;

/**
 * Checks the application id and version.
 * 
 * @author bufferings[at]gmail.com
 */
public class ApplicationChecker {

  private static final Logger logger =
    Logger.getLogger(ApplicationChecker.class.getName());

  protected static final String RESOURCE_NAME = "ApplicationChecker.properties";

  protected static final String APPLICATION_KEY = "application";

  protected static final String VERSION_KEY = "version";

  protected static Boolean condition = null;

  /**
   * Checks if the application fulfill the condition for running tests on the
   * production server.
   * 
   * @throws KtrS3UnitException
   *           when the condition is not fulfilled.
   */
  public void checkApplication() throws KtrS3UnitException {
    if (condition != null) {
      if (condition == Boolean.FALSE) {
        throw new KtrS3UnitException("Condition is not fulfilled.");
      } else {
        return;
      }
    }

    Properties props;
    try {
      props = loadProperties();
    } catch (Exception e) {
      condition = Boolean.FALSE;
      throw new KtrS3UnitException("Cannot load the condition resource.", e);
    }

    String propApplication = props.getProperty(APPLICATION_KEY);
    if (propApplication == null) {
      throw new AssertionError(APPLICATION_KEY + " key not found.");
    }

    String propVersion = props.getProperty(VERSION_KEY);
    if (propVersion == null) {
      throw new AssertionError(VERSION_KEY + " key not found.");
    }

    String appId = ApiProxy.getCurrentEnvironment().getAppId();
    String versionId = ApiProxy.getCurrentEnvironment().getVersionId();

    logger
      .info(String
        .format(
          "checkCondition[propApplication=%s, propVersion=%s, appId=%s, versionId=%s]",
          propApplication,
          propVersion,
          appId,
          versionId));

    if (!propApplication.equals(appId)) {
      condition = Boolean.FALSE;
      throw new KtrS3UnitException("Application ID is invalid.");
    }

    // ServerType type = new ServerTypeChecker().getServerType();
    // if (type == ServerType.PRODUCTION_SERVER) {
    int pos = versionId.lastIndexOf(".");
    if (pos == -1 || !propVersion.equals(versionId.substring(0, pos))) {
      condition = Boolean.FALSE;
      throw new KtrS3UnitException("Version ID is invalid.");
    }
    // } else {
    // if (!propVersion.equals(versionId)) {
    // condition = Boolean.FALSE;
    // throw new KtrS3UnitException("Version ID is invalid.");
    // }
    // }

    condition = Boolean.TRUE;
  }

  /**
   * Loads property file.
   */
  protected Properties loadProperties() throws Exception {
    InputStream is = null;
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      is = cl.getResourceAsStream(RESOURCE_NAME);
      if (is != null) {
        Properties props = new Properties();
        props.load(is);
        return props;
      } else {
        throw new AssertionError(RESOURCE_NAME + " resource file not found.");
      }
    } finally {
      if (is != null) {
        try {
          is.close();
        } catch (Exception e) {
          // ignore
        }
      }
    }
  }
}
