/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.s3unit.util;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import bufferings.ktr.s3unit.exception.KtrS3UnitException;

import com.google.apphosting.api.ApiProxy;

/**
 * A library configurator class for tests running without a server.
 * 
 * <p>
 * This class loads the appengine-api-stubs.jar and the
 * appengine-local-runtime.jar from AppEngineSDK.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 * 
 */
public class NoServerLibraryConfigurator {

  protected static final String IMPL_DIR_NAME = "impl";

  protected static final String API_STUBS_LIB_NAME = "appengine-api-stubs.jar";

  protected static final String LOCAL_RUNTIME_LIB_NAME =
    "appengine-local-runtime.jar";

  /**
   * Constructs a new NoServerLibraryConfigurator.
   */
  public NoServerLibraryConfigurator() {
  }

  /**
   * Configures the local libraries.
   */
  public void configure() {
    ClassLoader classLoader = getClass().getClassLoader();
    if (!(classLoader instanceof URLClassLoader)) {
      throw new KtrS3UnitException(
        "The classLoader is not an instance of URLClassLoader.");
    }

    URLClassLoader urlClassLoader = (URLClassLoader) classLoader;
    List<URL> currentUrls = Arrays.asList(urlClassLoader.getURLs());

    List<URL> urlsToAdd = prepareURLsToAdd();
    for (URL url : urlsToAdd) {
      if (!currentUrls.contains(url)) {
        try {
          Method m =
            URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
          m.setAccessible(true);
          m.invoke(urlClassLoader, url);
        } catch (Exception e) {
          throw new KtrS3UnitException("Cannot addURL to classloader.", e);
        }
      }
    }
  }

  /**
   * Prepares the urls to add.
   * 
   * @return the url list to add
   */
  protected List<URL> prepareURLsToAdd() {
    // Get the AppEngine location from ApiProxy protection domain.
    // AppEngineSDK/lib/shared/appengine-local-runtime-shared.jar
    URL appEngineSharedLocation =
      ApiProxy.class.getProtectionDomain().getCodeSource().getLocation();

    File sharedDir =
      new File(appEngineSharedLocation.getFile()).getParentFile();
    File implDir = new File(sharedDir.getParent(), IMPL_DIR_NAME);

    List<URL> urls = new ArrayList<URL>();
    urls.add(getLibraryUrl(implDir, API_STUBS_LIB_NAME));
    urls.add(getLibraryUrl(implDir, LOCAL_RUNTIME_LIB_NAME));
    return urls;
  }

  /**
   * Gets the library url from the directry and the filename.
   * 
   * @param dir
   *          the directory
   * @param libFileName
   *          the library name
   * @return the library url
   */
  protected URL getLibraryUrl(File dir, String libFileName) {
    File file = new File(dir, libFileName);
    if (!file.exists()) {
      StringBuilder msg =
        new StringBuilder()
          .append("Unable to find ")
          .append(libFileName)
          .append(" in ")
          .append(file.getAbsolutePath());
      throw new KtrS3UnitException(msg.toString());
    }

    try {
      return file.toURI().toURL();
    } catch (MalformedURLException e) {
      StringBuilder msg =
        new StringBuilder().append("Unable to retrieve a URL for ").append(
          file.getAbsolutePath());
      throw new KtrS3UnitException(msg.toString());
    }
  }
}
