/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.s3unit.util;

import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * A logging configurator class for tests running without a server.
 * <p>
 * Notice that this class uses {@link LogManager} which cannot be used on the
 * production server.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 */
public class NoServerLoggingConfigurator {

  private static final Logger logger =
    Logger.getLogger(NoServerLoggingConfigurator.class.getName());

  /** configured or not */
  protected static boolean configured = false;

  /** a configuration resource file */
  protected String confName = "NoServerTestLogging.properties";

  /**
   * Configures the log manager.
   */
  public void configure() {
    if (configured) {
      return;
    }
    try {
      applyConfiguration();
    } catch (Exception e) {
      logger.log(Level.WARNING, e.getMessage(), e);
    } finally {
      configured = true;
    }
  }

  /**
   * Applys configuration to the log manager.
   * <p>
   * When it fails to read resource file, it uses
   * {@link LogManager#readConfiguration()}.
   * </p>
   */
  protected void applyConfiguration() throws Exception {
    InputStream is = null;
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      is = cl.getResourceAsStream(confName);
      if (is != null) {
        LogManager.getLogManager().readConfiguration(is);
      } else {
        logger
          .warning("Configuration resource file not found. So default configuration is used.");
        LogManager.getLogManager().readConfiguration();
      }
    } finally {
      if (is != null) {
        try {
          is.close();
        } catch (Exception e) {
          // ignore
        }
      }
    }
  }
}
