/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.s3unit.util;

import java.util.logging.Logger;

import javax.servlet.ServletContext;

import org.slim3.util.ServletContextLocator;

/**
 * A class to get the server type.
 * <p>
 * This class checks the server type with {@link ServletContextLocator} on the
 * presupposition that the application uses
 * {@link org.slim3.controller.FrontController FrontController}.
 * </p>
 * <p>
 * The obtained server type is cached for the later request.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 */
public class ServerTypeChecker {

  /**
   * A enumeration of the server type.
   * 
   * @author bufferings[at]gmail.com
   */
  public enum ServerType {
    /** runnning on the production server */
    PRODUCTION_SERVER,
    /** running on the development server */
    DEVELOPMENT_SERVER,
    /** running without the server */
    NO_SERVER
  }

  private static final Logger logger =
    Logger.getLogger(ServerTypeChecker.class.getName());

  protected static final String PRODUCTION_SERVERINFO_PREFIX =
    "Google App Engine/";

  protected static final String DEVELOPMENT_SERVERINFO_PREFIX =
    "Google App Engine Development/";

  /** a cache of the server type */
  protected static ServerType serverTypeCache = null;

  /**
   * Gets the server type.
   * 
   * @return the server type
   */
  public ServerType getServerType() {
    if (serverTypeCache != null) {
      return serverTypeCache;
    }

    ServletContext servletContext = ServletContextLocator.get();
    String serverInfo = null;
    if (servletContext == null) {
      serverTypeCache = ServerType.NO_SERVER;
    } else {
      serverInfo = servletContext.getServerInfo();
      if (serverInfo.startsWith(PRODUCTION_SERVERINFO_PREFIX)) {
        serverTypeCache = ServerType.PRODUCTION_SERVER;
      } else if (serverInfo.startsWith(DEVELOPMENT_SERVERINFO_PREFIX)) {
        serverTypeCache = ServerType.DEVELOPMENT_SERVER;
      } else {
        // considers an unknown server as a development one
        serverTypeCache = ServerType.DEVELOPMENT_SERVER;
      }
    }

    logger.config(String.format(
      "Running on the %s. (ServerInfo = %s)",
      serverTypeCache,
      serverInfo));
    return serverTypeCache;
  }
}
